package org.apache.bcel.verifier.structurals;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.bcel.generic.ATHROW;
import org.apache.bcel.generic.BranchInstruction;
import org.apache.bcel.generic.GotoInstruction;
import org.apache.bcel.generic.Instruction;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.JsrInstruction;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.RET;
import org.apache.bcel.generic.ReturnInstruction;
import org.apache.bcel.generic.Select;
import org.apache.bcel.verifier.exc.AssertionViolatedException;
import org.apache.bcel.verifier.exc.StructuralCodeConstraintException;

/* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/ControlFlowGraph.class */
public class ControlFlowGraph {
    private final Subroutines subroutines;
    private final ExceptionHandlers exceptionhandlers;
    private final Map<InstructionHandle, InstructionContext> instructionContexts;

    /* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/ControlFlowGraph$InstructionContextImpl.class */
    private class InstructionContextImpl implements InstructionContext {
        private int TAG;
        private final InstructionHandle instruction;
        private final Map<InstructionContext, Frame> inFrames;
        private final Map<InstructionContext, Frame> outFrames;
        private List<InstructionContext> executionPredecessors;

        public InstructionContextImpl(InstructionHandle inst) {
            if (inst == null) {
                throw new AssertionViolatedException("Cannot instantiate InstructionContextImpl from NULL.");
            }
            this.instruction = inst;
            this.inFrames = new HashMap();
            this.outFrames = new HashMap();
        }

        private InstructionHandle[] _getSuccessors() {
            InstructionHandle[] single = new InstructionHandle[1];
            JsrInstruction instruction = getInstruction().getInstruction();
            if (instruction instanceof RET) {
                Subroutine s = ControlFlowGraph.this.subroutines.subroutineOf(getInstruction());
                if (s == null) {
                    throw new AssertionViolatedException("Asking for successors of a RET in dead code?!");
                }
                InstructionHandle[] jsrs = s.getEnteringJsrInstructions();
                InstructionHandle[] ret = new InstructionHandle[jsrs.length];
                Arrays.setAll(ret, i -> {
                    return jsrs[i].getNext();
                });
                return ret;
            }
            if ((instruction instanceof ReturnInstruction) || (instruction instanceof ATHROW)) {
                return InstructionHandle.EMPTY_ARRAY;
            }
            if (instruction instanceof JsrInstruction) {
                single[0] = instruction.getTarget();
                return single;
            }
            if (instruction instanceof GotoInstruction) {
                single[0] = ((GotoInstruction) instruction).getTarget();
                return single;
            }
            if (instruction instanceof BranchInstruction) {
                if (instruction instanceof Select) {
                    InstructionHandle[] matchTargets = ((Select) instruction).getTargets();
                    InstructionHandle[] ret2 = new InstructionHandle[matchTargets.length + 1];
                    ret2[0] = ((Select) instruction).getTarget();
                    System.arraycopy(matchTargets, 0, ret2, 1, matchTargets.length);
                    return ret2;
                }
                InstructionHandle[] pair = {getInstruction().getNext(), ((BranchInstruction) instruction).getTarget()};
                return pair;
            }
            single[0] = getInstruction().getNext();
            return single;
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public boolean execute(Frame inFrame, ArrayList<InstructionContext> execPreds, InstConstraintVisitor icv, ExecutionVisitor ev) {
            List<InstructionContext> clone = (List) execPreds.clone();
            this.executionPredecessors = clone;
            if ((lastExecutionJSR() == null && ControlFlowGraph.this.subroutines.subroutineOf(getInstruction()) != ControlFlowGraph.this.subroutines.getTopLevel()) || (lastExecutionJSR() != null && ControlFlowGraph.this.subroutines.subroutineOf(getInstruction()) == ControlFlowGraph.this.subroutines.getTopLevel())) {
                throw new AssertionViolatedException("Huh?! Am I '" + this + "' part of a subroutine or not?");
            }
            Frame inF = this.inFrames.get(lastExecutionJSR());
            if (inF == null) {
                this.inFrames.put(lastExecutionJSR(), inFrame);
                inF = inFrame;
            } else if (inF.equals(inFrame) || !mergeInFrames(inFrame)) {
                return false;
            }
            Frame workingFrame = inF.getClone();
            try {
                icv.setFrame(workingFrame);
                getInstruction().accept(icv);
                ev.setFrame(workingFrame);
                getInstruction().accept(ev);
                this.outFrames.put(lastExecutionJSR(), workingFrame);
                return true;
            } catch (StructuralCodeConstraintException ce) {
                ce.extendMessage("", "\nInstructionHandle: " + getInstruction() + "\n");
                ce.extendMessage("", "\nExecution Frame:\n" + workingFrame);
                extendMessageWithFlow(ce);
                throw ce;
            }
        }

        private void extendMessageWithFlow(StructuralCodeConstraintException e) {
            e.extendMessage("", "Execution flow:\n" + getExecutionChain());
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public ExceptionHandler[] getExceptionHandlers() {
            return ControlFlowGraph.this.exceptionhandlers.getExceptionHandlers(getInstruction());
        }

        private String getExecutionChain() {
            StringBuilder s = new StringBuilder(toString());
            for (int i = this.executionPredecessors.size() - 1; i >= 0; i--) {
                s.insert(0, this.executionPredecessors.get(i) + "\n");
            }
            return s.toString();
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public Frame getInFrame() {
            InstructionContext jsr = lastExecutionJSR();
            Frame org2 = this.inFrames.get(jsr);
            if (org2 == null) {
                throw new AssertionViolatedException("inFrame not set! This:\n" + this + "\nInFrames: '" + this.inFrames + "'.");
            }
            return org2.getClone();
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public InstructionHandle getInstruction() {
            return this.instruction;
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public Frame getOutFrame(ArrayList<InstructionContext> execChain) {
            this.executionPredecessors = execChain;
            InstructionContext jsr = lastExecutionJSR();
            Frame org2 = this.outFrames.get(jsr);
            if (org2 == null) {
                throw new AssertionViolatedException("outFrame not set! This:\n" + this + "\nExecutionChain: " + getExecutionChain() + "\nOutFrames: '" + this.outFrames + "'.");
            }
            return org2.getClone();
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public InstructionContext[] getSuccessors() {
            return ControlFlowGraph.this.contextsOf(_getSuccessors());
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public int getTag() {
            return this.TAG;
        }

        private InstructionContextImpl lastExecutionJSR() {
            int size = this.executionPredecessors.size();
            int retcount = 0;
            for (int i = size - 1; i >= 0; i--) {
                InstructionContextImpl current = (InstructionContextImpl) this.executionPredecessors.get(i);
                Instruction currentlast = current.getInstruction().getInstruction();
                if (currentlast instanceof RET) {
                    retcount++;
                }
                if (currentlast instanceof JsrInstruction) {
                    retcount--;
                    if (retcount == -1) {
                        return current;
                    }
                }
            }
            return null;
        }

        private boolean mergeInFrames(Frame inFrame) {
            Frame inF = this.inFrames.get(lastExecutionJSR());
            OperandStack oldstack = inF.getStack().getClone();
            LocalVariables oldlocals = inF.getLocals().getClone();
            try {
                inF.getStack().merge(inFrame.getStack());
                inF.getLocals().merge(inFrame.getLocals());
                return (oldstack.equals(inF.getStack()) && oldlocals.equals(inF.getLocals())) ? false : true;
            } catch (StructuralCodeConstraintException sce) {
                extendMessageWithFlow(sce);
                throw sce;
            }
        }

        @Override // org.apache.bcel.verifier.structurals.InstructionContext
        public void setTag(int tag) {
            this.TAG = tag;
        }

        public String toString() {
            return getInstruction().toString(false) + "\t[InstructionContext]";
        }
    }

    public ControlFlowGraph(MethodGen methodGen) {
        this(methodGen, true);
    }

    public ControlFlowGraph(MethodGen methodGen, boolean enableJustIceCheck) {
        this.instructionContexts = new HashMap();
        this.subroutines = new Subroutines(methodGen, enableJustIceCheck);
        this.exceptionhandlers = new ExceptionHandlers(methodGen);
        InstructionHandle[] instructionhandles = methodGen.getInstructionList().getInstructionHandles();
        for (InstructionHandle instructionhandle : instructionhandles) {
            this.instructionContexts.put(instructionhandle, new InstructionContextImpl(instructionhandle));
        }
    }

    public InstructionContext contextOf(InstructionHandle inst) {
        InstructionContext ic = this.instructionContexts.get(inst);
        if (ic == null) {
            throw new AssertionViolatedException("InstructionContext requested for an InstructionHandle that's not known!");
        }
        return ic;
    }

    public InstructionContext[] contextsOf(InstructionHandle[] insts) {
        InstructionContext[] ret = new InstructionContext[insts.length];
        Arrays.setAll(ret, i -> {
            return contextOf(insts[i]);
        });
        return ret;
    }

    public InstructionContext[] getInstructionContexts() {
        InstructionContext[] ret = new InstructionContext[this.instructionContexts.size()];
        return (InstructionContext[]) this.instructionContexts.values().toArray(ret);
    }

    public boolean isDead(InstructionHandle i) {
        return this.subroutines.subroutineOf(i) == null;
    }
}
